// $Id: simple_cck.js,v 1.1 2009/08/01 21:17:20 zserno Exp $

if (Drupal.jsEnabled) {
  $(function(){
    // Create tabs.
    $("#simple-cck-tabs>form>div").tabs();

    // Make fields draggable.
    $("#simple-cck-field-palette ul.simple-cck-fields li").draggable({
      helper: "clone",
      handle: "span"
    });

    // Make fields droppable.
    $("#simple-cck-drop-area-inner").droppable({
      accept: ".ui-draggable",
      activeClass: "simple-cck-drop-area-active",
      hoverClass: "simple-cck-drop-area-hover",
      drop: simpleCCKDrop
    });

    // Attach submit handler to form.
    $("#simple-cck-edit-form").submit(simpleCCKSubmit);

    // Store all the settings for all fields from the fieldlist.
    Drupal.settings.SimpleCCK.fields = new Object();

    // Build fieldlist from existing fields.
    if (Drupal.settings.SimpleCCK.existingFields) {
      var content = "";
      $.each(Drupal.settings.SimpleCCK.existingFields, function(fieldId, field) {
        _renderFieldList(field, fieldId, field.fieldType, true);
      });
    }

/******************************************************************************/

    // Scroll the 'Available fields' block with the window.
    // Heavily borrowed from Form Builder by quicksketch.
    var list = $('ul.simple-cck-fields');

    if (list.size()) {
      var block = list.parents('div.block:first').css('position', 'relative');
      var blockScrollStart = block.offset().top;

      function blockScroll() {
        var windowOffset = $(window).scrollTop();
        if (windowOffset - blockScrollStart > 0) {
          block.animate({ top: (windowOffset - blockScrollStart + 20) + 'px' }, 'fast');
        }
        else {
          block.animate({ top: '0px' }, 'fast');
        }
      }

      var timeout = false;
      function scrollTimeout() {
        if (timeout) {
          clearTimeout(timeout);
        }
        timeout = setTimeout(blockScroll, 100);
      }

      $(window).scroll(scrollTimeout);
    }
  });
}

/******************************************************************************/

/**
  * Drop event for a field.
  * @param event
  *   The original browser event.
  * @param ui
  *   A prepared ui object.
  */
function simpleCCKDrop(event, ui) {
  _addNewField(ui.draggable.attr("title"));
}

/**
  * Submit handler function.
  * Iterates through fieldlist, parses information,
  * stores in Drupal.settings.SimpleCCK.fields object then sends to the server.
  */
function simpleCCKSubmit() {
  $.each($("#simple-cck-field-list li"), function(i, v) {
    var obj = new Object();
    var parentId = _convertHyphens($(this).attr("id"));

    obj.field_widget_type = $(this).attr("title");
    obj.weight = i;

    // Walk through configuration input fields.
    $.each($(this).find(".simple-cck-textfield input"), function() {
      // Take care of risky characters.
      val = $(this).val().replace(/[\'\"]/gi, '&#39;');
      eval('obj.' + $(this).attr("title") + ' = "' + val + '";');
    });

    $.each($(this).find(".simple-cck-checkbox input"), function() {
      var checked = (this.checked) ? 1 : 0;
      eval('obj.' + $(this).attr("title") + ' = "' + checked + '";');
    });

    // Walk through configuration textareas.
    $.each($(this).find(".simple-cck-textarea textarea"), function() {
      var val = $(this).val().replace(/\n/gi, "\\n");
      eval('obj.' + $(this).attr("title") + ' = "' + val + '";');
    });

    // Walk through configuration checkboxes.
    var options = new Array();
    $.each($(this).find(".simple-cck-checkboxes input"), function() {
      if (this.checked) {
        var val = this.value.replace('prefix_', '');
        options.push(val);
      }
    });

    var type = $(this).attr("title");
    // Check field_widget_type and only add options property to obj if it's a reference field.
    if (type =='nodereference-nodereference_autocomplete' || type =='userreference-userreference_autocomplete') {
      var property = $(this).find("div[@class*=simple-cck-checkboxes]").attr("title");
      eval('obj.' + property + ' = "' + options + '";');
    }

    // Register field.
    eval('Drupal.settings.SimpleCCK.fields.' + parentId + ' = obj;');
  });
  // Convert to JSON, so we can send it to server.
  var fieldsInJSON = $.toJSON(Drupal.settings.SimpleCCK.fields);
  $("#simple-cck-drop-area").prepend('<input type="hidden" name="simple_cck_fields" value=\'' + fieldsInJSON + '\' />');
}

/******************************************************************************/

/**
  * Private function to add a new field to the field list.
  * @param fieldType
  *   String, type of the field. e.g. 'textfield'
  */
function _addNewField(fieldType) {
  var field = eval('Drupal.settings.SimpleCCK.fieldTypes.' + fieldType);
  var fieldList = $("#simple-cck-field-list");
  var numberOfFields = fieldList.children().length;
  var fieldId = 'simple-cck-field-' + numberOfFields;

  _renderFieldList(field, fieldId, fieldType, false);
}

/**
  * Private function to render the field list.
  * @param field
  *   A field object.
  *   e.g. textfield, textarea.
  * @param fieldId
  *   String, id attribute of the current field's <li> tag.
  *   e.g. simple-cck-field-0.
  * @param fieldType
  *   String, type of the field. e.g. 'textfield'
  * @param existing

  *   Boolean, indicates whether the field to be rendered is an existing field.
  */
function _renderFieldList(field, fieldId, fieldType, existing) {
  var fieldList = $("#simple-cck-field-list");
  // Check if first field was added.
  var initText = $(".simple-cck-drop-area-init-text");
  if (initText.length) {
    initText.replaceWith('<ul id="simple-cck-field-list"></ul>');
    fieldList = $("#simple-cck-field-list");
    // Make them sortable.
    fieldList.sortable({
      handle: 'a.simple-cck-drag-handler',
      revert: true,
      axis: 'y',
      containment: '#simple-cck-drop-area-inner',
      placeholder: 'simple-cck-field-list-highlight'
    });
  }

  var content = _renderConfiguration(field, fieldId, existing);
  var widget = field.type + '-' + field.widget.type;
  var fieldHTML = '<li id="' + fieldId + '" title="' + widget + '" class="simple-cck-' + fieldType + '"><a class="simple-cck-drag-handler"></a>' + content + '</li>';

  // Convert text to jQuery object
  $(fieldHTML)
  // Add to field list.
  .appendTo(fieldList)
  // Update sortable.
  .sortable('refresh');

  // Attach click handler to 'Configure' link.
  $("#" + fieldId + " .simple-cck-field-configure").click(function() {
    var active = $(".simple-cck-configure-active"); // Currently opened field.
    var activeId = active.parents("li[@class^=simple-cck-]").attr("id"); // Id of opened field.

    // If clicked on a non-active Configure link, close the active one.
    if (activeId != fieldId) {
      active.slideToggle("fast").toggleClass('simple-cck-configure-active');
    }
    $(this).next().next().next().slideToggle("fast").toggleClass('simple-cck-configure-active');
  });

  // Attach click handler to 'Remove' link.
  $("#" + fieldId + " .simple-cck-field-remove").click(function() {
    $("#" + fieldId).fadeOut("fast", function() {
      $(this).remove();
    });
  });

  // Attach keyup handler to 'Label' field.
  $("#" + fieldId + "-label").keyup(function() {
    $("#" + fieldId + " .simple-cck-field-label").text($(this).val());
  });
}

/**
  * Private function to render a field in the fieldlist.
  * @param field
  *   A field object.
  *   e.g. textfield, textarea.
  * @param fieldId
  *   String, id attribute of the current field's <li> tag.
  *   e.g. simple-cck-field-0.
  * @param existing
  *   Boolean, indicates whether the field to be rendered is an existing field.
  * @return
  *   The rendered content.
  */
function _renderConfiguration(field, fieldId, existing) {
  // Check if we are about to render an existing field.
  var existingField = {general: null, widget: null};
  if (existing) {
    existingField = field;
    field = eval('Drupal.settings.SimpleCCK.fieldTypes.' + field.type + '_' + field.widget.type);
  }
  var label = (existing) ? existingField.general.label : field.general.label.default_value;
  // Generate content of the field.
  var content = '<span class="simple-cck-field-title">' + field.title + '</span>';
  content += '<span class="simple-cck-field-label">' + label + '</span>';
  content += '<span class="simple-cck-field-configure"><a>Configure</a></span>';
  content += '<span class="simple-cck-field-remove"><a title="Remove this field.">[x]</a></span>';
  content += '<br class="clear-left" />';
  content += '<div class="simple-cck-field-configuration-wrapper">';

  $.each(field.widget, function(key, value) {
    if ((typeof value) == 'object') {
      var configId = fieldId + '-' + key;
      content += _renderConfigurationElement(value, configId, key, existingField.widget);
    }
  });

  $.each(field.general, function(key, value) {
    var configId = fieldId + '-' + key;
    content += _renderConfigurationElement(value, configId, key, existingField.general);
  });

  content += '</div>';

  return content;
}

/**
  * Private function to render a configuration form element.
  * @param config
  *   Object that contains elements such as title (label of the input field), default value, size, options...etc.
  *   We assume that config has these necessary properties.
  * @param id
  *   String, a unique id for the actual form element. Needed for <label> tags to reference an input element.
  * @param property
  *   Name of current property, used as title attribute.
  * @param existingField
  *
  * @return
  *   The rendered XHTML markup.
  */
function _renderConfigurationElement(config, id, property, existingField) {
  var output = "";
  var defaultValue = (existingField) ? eval('existingField.' + property) : config.default_value;
  switch (config.type) {
    case 'textfield':
      output = '<div class="simple-cck-field-configuration simple-cck-textfield">'
        + '<label for="' + id + '">' + config.title + ':</label>'
        + '<input type="text" id="' + id + '" title="' + property + '" value="' + defaultValue + '" size="' + config.size + '" />'
        + '</div>';
    break;

    case 'checkbox':
      output = '<div class="simple-cck-field-configuration simple-cck-checkbox">'
        + '<label for="' + id + '">' + config.title + ':</label>'
        + '<input type="checkbox" value="1" id="' + id + '" title="' + property + '"' + ((defaultValue) == 1 ? ' checked="checked"' : '') + ' />'
        + '</div>';
    break;

    case 'textarea':
      output = '<div class="simple-cck-field-configuration simple-cck-textarea">'
        + '<label for="' + id + '">' + config.title + ':</label>'
        + '<textarea id="' + id + '" title="' + property + '" rows="' + config.rows + '">' + defaultValue + '</textarea>'
        + '</div>';
    break;

    case 'checkboxes':
      output = '<div class="simple-cck-field-configuration simple-cck-checkboxes" title="' + property + '">' + config.title + ':';
      $.each(config.options, function(key, value){
        output += '<div class="form-item"><label class="option"><input type="checkbox" value="' + key + '"' + (eval('defaultValue.' + key) ? ' checked="checked"' : '') + ' />' + value + '</label></div>';
      });
      output += '</div>';
    break;
  }

  return output;
}

/**
  * Private function to convert id and class strings to a format,
  * which can be used as property of an object.
  * Basically replaces hyphens (-) with underscores (_).
  * @param str
  *   String to convert.
  * @return
  *   Converted string.
  */
function _convertHyphens(str) {
  return str.replace(/[-]+/g, '_');
}
